# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""GUI redirect server applications."""

import logging
import os

from concurrent import futures
from tornado import (
    gen,
    web,
)
from tornado.options import options

from guiredirect import (
    blocking,
    handlers,
    utils,
)


# Set up the main application settings.
_project_dir = os.path.abspath(os.path.dirname(__file__))
SETTINGS = {
    # Define the application static files path.
    'static_path': os.path.join(_project_dir, 'static'),
    # Define the application templates path.
    'template_path': os.path.join(_project_dir, 'templates'),
}


def server(gui_info):
    """Return the main server application."""
    handler_options = {'gui_info': gui_info, 'redirect_port': options.redirect}
    return web.Application([
        (r'.*', handlers.WaitAndRedirectHandler, handler_options),
    ], **SETTINGS)


@gen.coroutine
def watcher(gui_info):
    """Observe the Juju environment in a separate process.

    Properly change the information stored in the given gui_info object when
    the GUI becomes available.

    Also execute the provided external command when the GUI is ready.
    """
    logging.info('watching the Juju environment on {}'.format(options.apiurl))
    with futures.ProcessPoolExecutor(1) as executor:
        try:
            address = gui_info.address = yield executor.submit(
                blocking.wait_for_gui, options.apiurl, options.admin_secret)
        except Exception as err:
            gui_info.error = 'watcher error: {}'.format(err)
            logging.error(gui_info.error)
            return

    logging.info('the Juju GUI started ({})'.format(address))
    gui_info.started = True

    command = os.path.abspath(os.path.expanduser(options.command))
    logging.info('executing the external command {}'.format(command))
    retcode, output, error = yield utils.call_subprocess(command, address)
    logging.debug('external command output:\n{}'.format(output))
    if retcode:
        gui_info.error = 'subprocess error {}: {!r}'.format(retcode, error)
        logging.error(gui_info.error)
        return

    # Now taht the subprocess exited without errors, we assume the redirect
    # HTTP port is ready to serve the GUI.
    gui_info.ready = True
    logging.info('done: ready to redirect to port {}'.format(options.redirect))
