# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""GUI redirect server blocking functions.

The functions in this module use the blocking Python Juju Client library.
Therefore they must be called in a separate thread or process.
"""

import logging

import jujuclient


def _watch_changes(watcher):
    """Observe the Juju environment changes iterating over the given watcher.

    Raise a ValueError if the Juju GUI unit goes in an error state.
    Return the Juju GUI unit public address.
    """
    unit = 'juju-gui/0'
    for changeset in watcher:
        logging.debug('new changeset from Juju: {}'.format(changeset))
        for entity, action, data in changeset:
            if entity != 'unit' or action == 'remove' or data['Name'] != unit:
                continue
            status = data['Status']
            if 'error' in status:
                msg = 'juju-gui is in an error state: {}: {}'.format(
                    status, data['StatusInfo'])
                raise ValueError(msg)
            if status == 'started':
                return data['PublicAddress']
    raise ValueError('juju-gui public address not found')


def wait_for_gui(apiurl, admin_secret):
    """Wait for the Juju GUI to be ready and return the GUI machine address.

    This function is blocking and must be called in a separate process.
    """
    environment = jujuclient.Environment(apiurl)
    environment.login(admin_secret)
    try:
        with environment.watch() as watcher:
            return _watch_changes(watcher)
    finally:
        environment.close()
