# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""GUI redirect server management."""

import logging
import os
import sys

from tornado import util
from tornado.ioloop import IOLoop
from tornado.options import (
    define,
    options,
    parse_command_line,
)

import guiredirect
from guiredirect.apps import (
    server,
    utils,
    watcher,
)


DEFAULT_APIURL = 'wss://localhost:17070'
DEFAULT_ENVFILE = '~/.juju/environments.yaml'


def _validate_admin_secret():
    """Define an admin_secret options from the envname and envfile args.

    Exit with an error if the admin secret cannot be retrieved.
    """
    try:
        admin_secret = utils.get_admin_secret(options.envname, options.envfile)
    except ValueError as err:
        sys.exit('error: {}'.format(err))
    else:
        options.define('admin_secret', default=admin_secret)


def _validate_different(arg1, arg2):
    """Check that the values associated with the given arguments are different.

    Exit with an error otherwise.
    """
    if options[arg1] == options[arg2]:
        error = 'error: the same value cannot be assigned to {} and {}'.format(
            arg1, arg2)
        sys.exit(error)


def _validate_executable(arg):
    """Ensure the given path points to an existing executable file.

    Exit with an error otherwise.
    """
    path = os.path.expanduser(options[arg])
    if not os.path.isfile(path):
        sys.exit('error: {} argument: no such file {}'.format(arg, path))
    elif not os.access(path, os.X_OK):
        sys.exit('error: {} argument: {} is not executable'.format(arg, path))


def _validate_required(*args):
    """Validate required arguments.

    Exit with an error if a mandatory argument is missing.
    """
    for name in args:
        value = options[name]
        if value is None or not str(value).strip():
            sys.exit('error: the {} argument is required'.format(name))


def setup():
    """Set up options and logger."""
    define(
        'command', type=str,
        help='The external command/executable to run when the Juju GUI is '
             'ready/started. The command is called passing the Juju GUI unit '
             'public address as first argument. The server starts redirecting '
             'to the port specified in --redirect after the command exited '
             'without errors.')
    define(
        'envname', type=str, help='The name of the Juju environment to use.')
    define(
        'listen', type=int,
        help='The HTTP port where the server will start listening.')
    define(
        'redirect', type=int,
        help='The HTTP port where the server will start redirecting requests '
             'when the Juju GUI is ready.')
    # Optional parameters.
    define(
        'apiurl', type=str, default=DEFAULT_APIURL,
        help='The Juju WebSocket server address. This is usually the address '
             'of the bootstrap/state node as returned by "juju status".')
    define(
        'envfile', type=str, default=DEFAULT_ENVFILE,
        help='The path to the Juju environments YAML file.')
    # In Tornado, parsing the options also sets up the default logger.
    parse_command_line()
    _validate_required('command', 'envname', 'listen', 'redirect')
    _validate_different('listen', 'redirect')
    _validate_executable('command')
    _validate_admin_secret()


def run():
    """Run the server"""
    # Set up an object that will be used to store information about the GUI.
    # The gui_info instance will be mutated by the watcher and checked by the
    # HTTP server.
    gui_info = util.ObjectDict(
        address=None,
        error=None,
        ready=False,
        started=False,
    )
    version = guiredirect.get_version()
    logging.info('starting GUI redirect server v{}'.format(version))
    logging.info('listening on port {}'.format(options.listen))
    # Start the HTTP server and the Juju environment watcher.
    server(gui_info).listen(options.listen)
    watcher(gui_info)
    IOLoop.instance().start()
