# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the GUI redirect server blocking functions."""

import unittest

import mock

from guiredirect import blocking
from guiredirect.tests import helpers


class TestWatchChanges(helpers.ValueErrorTestsMixin, unittest.TestCase):

    def test_error_address_not_found(self):
        # A ValueError is raised if the unit public address is not found.
        changesets = (
            [('unit', 'change', {'Name': 'django/0'})],  # Invalid unit data.
            [('unit', 'remove', {'Name': 'juju-gui/0'})],  # Invalid action.
            [('service', 'change', {'Name': 'juju-gui/0'})],  # invalid entity.
        )
        with self.assert_value_error('juju-gui public address not found'):
            blocking._watch_changes(changesets)

    def test_error_no_changesets(self):
        # A ValueError is raised if the given watcher has no changes.
        with self.assert_value_error('juju-gui public address not found'):
            blocking._watch_changes([])

    def test_not_started(self):
        # A ValueError is raised if the unit never started.
        changeset = (
            ('unit', 'add', {'Name': 'juju-gui/0', 'Status': 'pending'}),
            ('unit', 'change', {'Name': 'juju-gui/0', 'Status': 'installed'}),
        )
        with self.assert_value_error('juju-gui public address not found'):
            blocking._watch_changes([changeset])

    def test_status_error(self):
        # A ValueError is raised if the unit goes in an error state.
        changeset = [(
            'unit', 'add', {
                'Name': 'juju-gui/0',
                'Status': 'error',
                'StatusInfo': 'start hook',
            }
        )]
        expected_error = 'juju-gui is in an error state: error: start hook'
        with self.assert_value_error(expected_error):
            blocking._watch_changes([changeset])

    def test_address_found(self):
        # The unit public address is correctly returned.
        changeset = (
            ('unit', 'add', {'Name': 'juju-gui/0', 'Status': 'pending'}),
            ('unit', 'change', {'Name': 'juju-gui/0', 'Status': 'installed'}),
            ('unit', 'change', {
                'Name': 'juju-gui/0',
                'Status': 'started',
                'PublicAddress': 'example.com',
            }),
        )
        address = blocking._watch_changes([changeset])
        self.assertEqual('example.com', address)


@mock.patch('guiredirect.blocking.jujuclient.Environment')
@mock.patch('guiredirect.blocking._watch_changes')
class TestWaitForGui(helpers.ValueErrorTestsMixin, unittest.TestCase):

    apiurl = 'wss://api.example.com:17070'
    admin_secret = 'Secret!'

    def test_environment_calls(self, mock_watch_changes, mock_environment):
        # The Juju Client Environment object is correctly used.
        blocking.wait_for_gui(self.apiurl, self.admin_secret)
        # The environment has been instantiated.
        mock_environment.assert_called_once_with(self.apiurl)
        mock_instance = mock_environment()
        # The connection has been authenticated.
        mock_instance.login.assert_called_once_with(self.admin_secret)
        # We started watching the environment.
        mock_instance.watch.assert_called_once_with()
        # The Juju connection is correctly closed at the end of the process.
        mock_instance.close.assert_called_once_with()

    def test_failure(self, mock_watch_changes, mock_environment):
        # A ValueError is raised if an error occurs while watching the Juju
        # environment.
        mock_watch_changes.side_effect = ValueError('bad wolf')
        with self.assert_value_error('bad wolf'):
            blocking.wait_for_gui(self.apiurl, self.admin_secret)
        # The Juju connection is correctly closed at the end of the process.
        mock_environment().close.assert_called_once_with()

    def test_success(self, mock_watch_changes, mock_environment):
        # The unit public address is correctly returned.
        mock_watch_changes.return_value = 'example.com'
        address = blocking.wait_for_gui(self.apiurl, self.admin_secret)
        self.assertEqual('example.com', address)
        # The Juju connection is correctly closed at the end of the process.
        mock_environment().close.assert_called_once_with()
