# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the GUI redirect server handlers."""

from tornado import (
    util,
    web,
)
from tornado.testing import (
    AsyncHTTPTestCase,
    LogTrapTestCase,
)

from guiredirect import (
    apps,
    handlers,
)


class TestWaitAndRedirectHandler(LogTrapTestCase, AsyncHTTPTestCase):

    def get_app(self):
        self.gui_info = util.ObjectDict(error=None, ready=False)
        self.redirect_port = 42
        handler_options = {
            'gui_info': self.gui_info,
            'redirect_port': self.redirect_port,
        }
        return web.Application([
            (r'.*', handlers.WaitAndRedirectHandler, handler_options),
        ], **apps.SETTINGS)

    def assert_redirected(self, response, path):
        """Ensure the given response is a permanent redirect to the given path.

        Also check that the URL schema is HTTPS.
        """
        self.assertEqual(301, response.code)
        expected = 'http://localhost:{}{}'.format(self.redirect_port, path)
        self.assertEqual(expected, response.headers['location'])

    def test_gui_not_ready(self):
        # The placeholder page include the error if something wrong happened
        # watching the Juju environment.
        self.gui_info.error = 'bad wolf'
        response = self.fetch('/')
        self.assertEqual(200, response.code)
        self.assertIn('bad wolf', response.body)

    def test_gui_in_error(self):
        # A placeholder page is served if the Juju GUI is not yet ready.
        response = self.fetch('/')
        self.assertEqual(200, response.code)
        self.assertIn('Juju GUI', response.body)

    def test_redirection(self):
        # The HTTP(S) traffic is redirected to the given port.
        self.gui_info.ready = True
        response = self.fetch('/', follow_redirects=False)
        self.assert_redirected(response, '/')

    def test_page_redirection(self):
        # The path and query parts of the URL are preserved.
        self.gui_info.ready = True
        path_and_query = '/my/page?my=query'
        response = self.fetch(path_and_query, follow_redirects=False)
        self.assert_redirected(response, path_and_query)
