# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the GUI redirect server management infrastructure."""

from contextlib import contextmanager
import os
import sys
import unittest

import mock

from guiredirect import manage


class ValidatorTestMixin(object):
    """Add methods for testing functions producing a system exit."""

    @contextmanager
    def assert_sysexit(self, error):
        """Ensure the code in the context manager block produces a system exit.

        Also check that the given error is returned.
        """
        with mock.patch('sys.exit') as mock_exit:
            yield
            mock_exit.assert_called_once_with(error)


@mock.patch('guiredirect.manage.options')
@mock.patch('guiredirect.manage.utils.get_admin_secret')
class TestValidateAdminSecret(ValidatorTestMixin, unittest.TestCase):

    error = 'error: bad wolf'

    def test_success(self, mock_get_admin_secret, mock_options):
        # The validation passes if the admin_secret is correctly retrieved.
        mock_get_admin_secret.return_value = 'Secret!'
        manage._validate_admin_secret()
        mock_options.define.assert_called_once_with(
            'admin_secret', default='Secret!')

    def test_failure(self, mock_get_admin_secret, mock_options):
        # The validation failse if problems occurred while trying to retrieve
        # the Juju admin_secret.
        mock_get_admin_secret.side_effect = ValueError('bad wolf')
        with self.assert_sysexit('error: bad wolf'):
            manage._validate_admin_secret()


class TestValidateDifferent(ValidatorTestMixin, unittest.TestCase):

    error = 'error: the same value cannot be assigned to {} and {}'

    def test_success(self):
        # The validation passes if the args are different.
        options = {'arg1': 'value1', 'arg2': 'value2'}
        with mock.patch('guiredirect.manage.options', options):
            manage._validate_different('arg1', 'arg2')

    def test_failure(self):
        # The validation failse if the args store the same value.
        options = {'arg1': 'value1', 'arg2': 'value1'}
        with mock.patch('guiredirect.manage.options', options):
            with self.assert_sysexit(self.error.format('arg1', 'arg2')):
                manage._validate_different('arg1', 'arg2')


class TestValidateExecutable(ValidatorTestMixin, unittest.TestCase):

    def test_success(self):
        # The validation passes if the argument points to an existing
        # executable file.
        options = {'arg1': sys.executable}
        with mock.patch('guiredirect.manage.options', options):
            manage._validate_executable('arg1')

    def test_success_expanduser(self):
        # The validation passes if the argument points to an existing
        # executable file, expandingf the ~ construct.
        options = {'arg1': '~/../../../../' + sys.executable}
        with mock.patch('guiredirect.manage.options', options):
            manage._validate_executable('arg1')

    def test_not_found(self):
        # The validation fails if the file does not exist.
        options = {'arg1': '/no/such/file'}
        error = 'error: arg1 argument: no such file /no/such/file'
        with mock.patch('guiredirect.manage.options', options):
            with self.assert_sysexit(error):
                manage._validate_executable('arg1')

    def test_not_executable(self):
        # The validation fails if the file is not executable.
        options = {'arg1': __file__}
        error = 'error: arg1 argument: {} is not executable'
        with mock.patch('guiredirect.manage.options', options):
            with self.assert_sysexit(error.format(os.path.abspath(__file__))):
                manage._validate_executable('arg1')


class TestValidateRequired(ValidatorTestMixin, unittest.TestCase):

    error = 'error: the {} argument is required'

    def test_success(self):
        # The validation passes if the args are correctly found.
        with mock.patch('guiredirect.manage.options', {'arg1': 'value1'}):
            manage._validate_required('arg1')

    def test_success_multiple_args(self):
        # The validation passes for multiple args.
        options = {'arg1': 'value1', 'arg2': 'value2'}
        with mock.patch('guiredirect.manage.options', options):
            manage._validate_required(*options.keys())

    def test_success_not_string(self):
        # The validation passes if the arg value is not a string.
        with mock.patch('guiredirect.manage.options', {'arg1': 42}):
            manage._validate_required('arg1')

    def test_failure(self):
        # The validation fails if the arg value is an empty string.
        with mock.patch('guiredirect.manage.options', {'arg1': ''}):
            with self.assert_sysexit(self.error.format('arg1')):
                manage._validate_required('arg1')

    def test_failure_multiple_args(self):
        # The validation fails if one of the arg values is an empty string.
        options = {'arg1': 'value1', 'arg2': ''}
        with mock.patch('guiredirect.manage.options', options):
            with self.assert_sysexit(self.error.format('arg2')):
                manage._validate_required(*options.keys())

    def test_failure_missing(self):
        # The validation fails if the value is missing.
        with mock.patch('guiredirect.manage.options', {'arg1': None}):
            with self.assert_sysexit(self.error.format('arg1')):
                manage._validate_required('arg1')

    def test_failure_empty(self):
        # The validation fails if the stripped value is an empty string.
        with mock.patch('guiredirect.manage.options', {'arg1': ' '}):
            with self.assert_sysexit(self.error.format('arg1')):
                manage._validate_required('arg1')
