# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the GUI redirect server helper functions and objects."""

import os
import shutil
import tempfile
import unittest

import mock
from tornado.testing import(
    AsyncTestCase,
    gen_test,
)
import yaml

from guiredirect import utils
from guiredirect.tests import helpers


class TestCallSubprocess(AsyncTestCase):

    @gen_test
    def test_success(self):
        # A zero exit code and the subprocess output are correctly returned.
        retcode, output, error = yield utils.call_subprocess(
            'echo', io_loop=self.io_loop)
        self.assertEqual(0, retcode)
        self.assertEqual('\n', output)
        self.assertEqual('', error)

    @gen_test
    def test_multiple_arguments(self):
        # A zero exit code and the subprocess output are correctly returned
        # when executing a command passing multiple arguments.
        retcode, output, error = yield utils.call_subprocess(
            'echo', 'we are the borg!', io_loop=self.io_loop)
        self.assertEqual(0, retcode)
        self.assertEqual('we are the borg!\n', output)
        self.assertEqual('', error)

    @gen_test
    def test_failure(self):
        # An error code and the error are returned if the subprocess fails.
        retcode, output, error = yield utils.call_subprocess(
            'ls', 'no-such-file', io_loop=self.io_loop)
        self.assertEqual(2, retcode)
        self.assertEqual('', output)
        self.assertEqual(
            'ls: cannot access no-such-file: No such file or directory\n',
            error)


class TestGetAdminSecret(helpers.ValueErrorTestsMixin, unittest.TestCase):

    def mock_environment_file(self, contents):
        """Create a mock environment file containing the given contents."""
        # Create a temporary home that will be used by get_admin_secret().
        home = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, home)
        # Create a juju home.
        juju_home = os.path.join(home, '.juju')
        os.mkdir(juju_home)
        # Set up an environments file with the given contents.
        self.environments_path = os.path.join(juju_home, 'environments.yaml')
        with open(self.environments_path, 'w') as environments_file:
            environments_file.write(contents)
        # Return a mock object patching the environment context with the
        # temporary HOME. The returned object can be used as a context manager.
        return mock.patch('os.environ', {'HOME': home})

    def test_no_file(self):
        # A ValueError is raised if the environments file is not found.
        expected = (
            'Unable to open environments file: [Errno 2] '
            "No such file or directory: '/no/such/file.yaml'"
        )
        with self.assert_value_error(expected):
            utils.get_admin_secret('ec2', '/no/such/file.yaml')

    def test_invalid_yaml(self):
        # A ValueError is raised if the environments file is not well formed.
        with self.mock_environment_file(':'):
            with self.assertRaises(ValueError) as context_manager:
                utils.get_admin_secret('ec2')
        error = str(context_manager.exception)
        self.assertIn('Unable to parse environments file:', error)

    def test_invalid_yaml_contents(self):
        # A ValueError is raised if the environments file is not well formed.
        with self.mock_environment_file('a-string'):
            with self.assert_value_error('Invalid YAML contents: a-string'):
                utils.get_admin_secret('ec2')

    def test_no_env(self):
        # A ValueError is raised if the environment is not found in the YAML.
        contents = yaml.safe_dump({'environments': {'local': {}}})
        with self.mock_environment_file(contents):
            with self.assert_value_error('Environment ec2 not found'):
                utils.get_admin_secret('ec2')

    def test_no_admin_secret(self):
        # A ValueError is raised if the admin secret is not included in the
        # environment info.
        contents = yaml.safe_dump({'environments': {'ec2': {}}})
        with self.mock_environment_file(contents):
            with self.assert_value_error('Admin secret not found'):
                utils.get_admin_secret('ec2')

    def test_success(self):
        # The environment admin secret is correctly returned.
        contents = yaml.safe_dump({
            'environments': {'ec2': {'admin-secret': 'Secret!'}},
        })
        with self.mock_environment_file(contents):
            self.assertEqual('Secret!', utils.get_admin_secret('ec2'))

    def test_success_customized_environment_file(self):
        # The environment admin secret is correctly returned from a customized
        # environment file.
        contents = yaml.safe_dump({
            'environments': {'ec2': {'admin-secret': 'Secret!'}},
        })
        with self.mock_environment_file(contents):
            admin_secret = utils.get_admin_secret(
                'ec2', envfile=self.environments_path)
        self.assertEqual('Secret!', admin_secret)
