# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""GUI redirect server helper functions and objects."""

import os
import subprocess

from tornado import process, gen
import yaml


@gen.coroutine
def call_subprocess(*args, **kwargs):
    """Call a subprocess asynchronously.

    Take the subcommand and its parameters as args, and all the
    subprocess.Popen kwargs. Also take an optional io_loop keyword argument,
    that will be passed to the underlying tornado.process.Subprocess instance.

    Return a Future whose result is a tuple containing the subprocess return
    code, output and error.

    See original at <http://tornadogists.org/6723392/>.
    """
    stream = process.Subprocess.STREAM
    params = {'stdin': subprocess.PIPE, 'stdout': stream, 'stderr': stream}
    params.update(kwargs)
    proc = process.Subprocess(args, **params)
    output, error = yield [
        gen.Task(proc.stdout.read_until_close),
        gen.Task(proc.stderr.read_until_close),
    ]
    # See tornado.process.Subprocess._set_returncode for the code below.
    _, status = os.waitpid(proc.pid, os.WNOHANG)
    if os.WIFSIGNALED(status):
        returncode = -os.WTERMSIG(status)
    else:
        assert os.WIFEXITED(status)
        returncode = os.WEXITSTATUS(status)
    raise gen.Return((returncode, output, error))


def get_admin_secret(envname, envfile='~/.juju/environments.yaml'):
    """Return the admin secret for the environment named envname.

    The envfile is the environments YAML file path, defaulting to
    ~/.juju/environments.yaml.

    Raise a ValueError if:
      - the environment file is not found;
      - the environment file contents are not parsable by YAML;
      - the YAML contents are not properly structured;
      - the environment named envname is not found;
      - the environment does not include the admin_secret field.
    """
    # Load and parse the Juju environments file.
    path = os.path.expanduser(envfile)
    try:
        environments_file = open(path)
    except IOError as err:
        raise ValueError('Unable to open environments file: {}'.format(err))
    try:
        environments = yaml.safe_load(environments_file)
    except Exception as err:
        raise ValueError('Unable to parse environments file: {}'.format(err))
    # Retrieve the admin secret for the current environment.
    try:
        environment = environments.get('environments', {}).get(envname)
    except AttributeError as err:
        raise ValueError('Invalid YAML contents: {}'.format(environments))
    if environment is None:
        raise ValueError('Environment {} not found'.format(envname))
    admin_secret = environment.get('admin-secret')
    if admin_secret is None:
        raise ValueError('Admin secret not found')
    return admin_secret
